package com.diagbot.facade;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.aggregate.AverageStatisticsAggregate;
import com.diagbot.aggregate.MrStatisticsAggregate;
import com.diagbot.aggregate.ResultStatisticsAggregate;
import com.diagbot.dto.AverageStatisticsDTO;
import com.diagbot.dto.CaseNumDTO;
import com.diagbot.dto.LevelStatisticsDTO;
import com.diagbot.dto.NumDTO;
import com.diagbot.dto.QcResultPercentDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.FilterOrderVO;
import com.diagbot.vo.FilterPageByAverageVO;
import com.diagbot.vo.FilterPageVO;
import com.diagbot.vo.FilterVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/4/13 16:45
 */
@Component
public class ConsoleFacade {
    @Autowired
    private FilterFacade filterFacade;
    @Autowired
    private DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;

    @Autowired
    private AverageStatisticsAggregate averageStatisticsAggregate;
    @Autowired
    private MrStatisticsAggregate mrStatisticsAggregate;
    @Autowired
    private ResultStatisticsAggregate resultStatisticsAggregate;
    @Autowired
    private QcresultInfoFacade qcresultInfoFacade;
    @Autowired
    private HomePageFacade homePageFacade;
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;

    //region-----------------------聚合接口开始-------------------------------

    /**
     * 病历相关统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> mrStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        filterVO.setHospitalId(hospitalId);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("filterVO", filterVO);
            retMap
                    = dataBeanAggregateQueryFacade.get("setAllMr", invokeParams, Map.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return retMap;
    }

    /**
     * 缺陷相关统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> resultStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setHospitalId(hospitalId);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("filterVO", filterVO);
            retMap
                    = dataBeanAggregateQueryFacade.get("setAllResult", invokeParams, Map.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return retMap;
    }

    /**
     * 平局值相关统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> averageStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setHospitalId(hospitalId);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("filterVO", filterVO);
            retMap
                    = dataBeanAggregateQueryFacade.get("setAllAverage", invokeParams, Map.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return retMap;
    }

    //endregion-----------------------聚合接口结束-------------------------------

    //region-----------------------单独接口开始-------------------------------

    /**
     * 平均住院天数
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getAverageDayNum(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<AverageStatisticsDTO> averageDayNumList = averageStatisticsAggregate.getAverageDayNum(filterVO);
        if (ListUtil.isNotEmpty(averageDayNumList)) {
            retMap.put("平均住院日", averageDayNumList);
        }
        return retMap;
    }

    /**
     * 平均住院费用
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getAverageFee(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<AverageStatisticsDTO> averageFeeList = averageStatisticsAggregate.getAverageFee(filterVO);
        if (ListUtil.isNotEmpty(averageFeeList)) {
            retMap.put("平均住院费用", averageFeeList);
        }
        return retMap;
    }

    /**
     * 质控平均分按科室统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getAverageScore(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<AverageStatisticsDTO> averageScoreList = averageStatisticsAggregate.getAverageScore(filterVO);
        if (ListUtil.isNotEmpty(averageScoreList)) {
            retMap.put("各科室质控平均分", averageScoreList);
        }
        return retMap;
    }

    /**
     * 各科室质控平均分（首页）-根据内外科系统统计
     *
     * @param filterVO
     * @return
     */
    public List<AverageStatisticsDTO> getAverageScoreByDeptClass(FilterVO filterVO) {
        filterVOSet(filterVO);
        List<AverageStatisticsDTO> retAverageScoreList = Lists.newLinkedList();
        List<AverageStatisticsDTO> averageScoreList = qcresultInfoFacade.getAverageScoreByDeptClass(filterVO);
        Integer limitCount = filterVO.getLimitCount();
        //质控平均分
        if (ListUtil.isNotEmpty(averageScoreList)) {
            retAverageScoreList = averageStatisticsAggregate.getLimitAverageList(averageScoreList, limitCount);
        }
        return retAverageScoreList;
    }

    /**
     * 各科室甲级病历占比
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getLevelResultDept(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<QcResultPercentDTO> levelResults = mrStatisticsAggregate.getLevelResultDept(filterVO);
        if (ListUtil.isNotEmpty(levelResults)) {
            retMap.put("各科室甲级病历占比", levelResults);
        }
        return retMap;
    }

    /**
     * 出院人数统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> leaveHosCount(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        Map<String, Object> leaveHosMap = mrStatisticsAggregate.leaveHosCount(filterVO);
        if (leaveHosMap != null) {
            retMap.put("出院人数统计", leaveHosMap);
        }
        return retMap;
    }

    /**
     * 质控病历统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> mrCount(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> mrCountList = mrStatisticsAggregate.mrCount(filterVO);
        if (ListUtil.isNotEmpty(mrCountList)) {
            retMap.put("病历数统计", mrCountList);
        }
        return retMap;
    }

    /**
     * 各模块缺陷占比排行
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> entryCountGroupByCase(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> caseList = resultStatisticsAggregate.entryCountGroupByCase(filterVO);
        if (ListUtil.isNotEmpty(caseList)) {
            retMap.put("各模块缺陷占比排行", caseList);
        }
        return retMap;
    }

    /**
     * 条目缺陷占比
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> entryCountGroupByEntry(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> entryList = resultStatisticsAggregate.entryCountGroupByEntry(filterVO);
        if (ListUtil.isNotEmpty(entryList)) {
            retMap.put("条目缺陷占比", entryList);
        }
        return retMap;
    }

    /**
     * 各科室缺陷占比
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> entryByDept(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> deptList = resultStatisticsAggregate.entryByDept(filterVO);
        if (ListUtil.isNotEmpty(deptList)) {
            retMap.put("各科室缺陷占比", deptList);
        }
        return retMap;
    }

    private void filterVOSet(FilterVO filterVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        filterVO.setHospitalId(hospitalId);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
    }
    //endregion-----------------------单独接口结束-------------------------------

    //region_______________________分页接口开始-------------------------------

    /**
     * 过滤条件设置
     *
     * @param filterPageVO
     */
    private void filterPageVOSet(FilterPageVO filterPageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = "";
        String endDate = "";
        Date date = new Date();

        //统计维度 1-本月，2-本年
        //时间类型 1-本月/本年，2-上月/去年，3-去年本月/去年
        if (filterPageVO.getType().equals(1)) {
            if (filterPageVO.getDateType() == null) {
                startDate = filterFacade.getStartDateStr(filterPageVO.getType(), null);
                endDate = filterFacade.getEndDateStr(filterPageVO.getType(), null);
            } else if (filterPageVO.getDateType().equals(2)) {
                startDate = filterFacade.getLastStartDateStr(filterPageVO.getType());
                endDate = filterFacade.getLastEndDateStr(filterPageVO.getType());
            } else if (filterPageVO.getDateType().equals(3)) {
                String year = DateUtil.getYear(date);
                startDate = filterFacade.getStartDateStr(filterPageVO.getType(), Integer.valueOf(year) - 1);
                endDate = filterFacade.getEndDateStr(filterPageVO.getType(), Integer.valueOf(year) - 1);
            } else {
                startDate = filterFacade.getStartDateStr(filterPageVO.getType(), null);
                endDate = filterFacade.getEndDateStr(filterPageVO.getType(), null);
            }
        } else if (filterPageVO.getType().equals(2)) {
            if (filterPageVO.getDateType() == null) {
                startDate = filterFacade.getStartDateStr(filterPageVO.getType(), null);
                endDate = filterFacade.getEndDateStr(filterPageVO.getType(), null);
            } else if (filterPageVO.getDateType().equals(2) || filterPageVO.getDateType().equals(3)) {
                startDate = filterFacade.getLastStartDateStr(filterPageVO.getType());
                endDate = filterFacade.getLastEndDateStr(filterPageVO.getType());
            } else {
                startDate = filterFacade.getStartDateStr(filterPageVO.getType(), null);
                endDate = filterFacade.getEndDateStr(filterPageVO.getType(), null);
            }
        }

        filterPageVO.setStartDate(startDate);
        filterPageVO.setEndDate(endDate);
        filterPageVO.setHospitalId(hospitalId);
    }

    /**
     * 平均住院天数、平均花费入参设置
     *
     * @param filterPageByAverageVO
     */
    private void filterPageByAverageVOSet(FilterPageByAverageVO filterPageByAverageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date date = new Date();
        String year = DateUtil.getYear(date);
        String startDate = filterFacade.getStartDateStr(filterPageByAverageVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterPageByAverageVO.getType(), null);
        String lastStartDate = filterFacade.getLastStartDateStr(filterPageByAverageVO.getType());
        String lastEndDate = filterFacade.getLastEndDateStr(filterPageByAverageVO.getType());
        String lastYearStartDate = filterFacade.getStartDateStr(filterPageByAverageVO.getType(), Integer.valueOf(year) - 1);
        String lastYearEndDate = filterFacade.getEndDateStr(filterPageByAverageVO.getType(), Integer.valueOf(year) - 1);
        filterPageByAverageVO.setHospitalId(hospitalId);
        filterPageByAverageVO.setUserId(Long.valueOf(userId));
        filterPageByAverageVO.setStartDate(startDate);
        filterPageByAverageVO.setEndDate(endDate);
        filterPageByAverageVO.setLastStartDate(lastStartDate);
        filterPageByAverageVO.setLastEndDate(lastEndDate);
        filterPageByAverageVO.setLastYearStartDate(lastYearStartDate);
        filterPageByAverageVO.setLastYearEndDate(lastYearEndDate);
    }

    /**
     * 各模块缺陷占比（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<NumDTO> entryCountGroupByCasePage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<NumDTO> page = qcresultInfoFacade.entryCountGroupByCasePage(filterPageVO);
        return page;
    }

    /**
     * 缺陷详情（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<CaseNumDTO> entryCountGroupByEntryPage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<CaseNumDTO> page = qcresultInfoFacade.entryCountGroupByEntryPage(filterPageVO);
        return page;
    }

    /**
     * 按科室统计平均住院天数（分页）
     *
     * @param filterPageByAverageVO
     * @return
     */
    public IPage<AverageStatisticsDTO> getAverageDayNumPage(FilterPageByAverageVO filterPageByAverageVO) {
        filterPageByAverageVOSet(filterPageByAverageVO);
        IPage<AverageStatisticsDTO> page = homePageFacade.getAverageDayNumPage(filterPageByAverageVO);
        return page;
    }

    /**
     * 按科室统计平均住院费用（分页）
     *
     * @param filterPageByAverageVO
     * @return
     */
    public IPage<AverageStatisticsDTO> getAverageFeePage(FilterPageByAverageVO filterPageByAverageVO) {
        filterPageByAverageVOSet(filterPageByAverageVO);
        IPage<AverageStatisticsDTO> page = homePageFacade.getAverageFeePage(filterPageByAverageVO);
        return page;
    }

    /**
     * 各科室质控平均分（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<AverageStatisticsDTO> getAverageScoreByDeptPage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<AverageStatisticsDTO> page = qcresultInfoFacade.getAverageScoreByDeptPage(filterPageVO);
        return page;
    }

    /**
     * 各科室缺陷占比排行（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<NumDTO> resultStatisticsByDeptPage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<NumDTO> page = behospitalInfoFacade.resultStatisticsByDeptPage(filterPageVO);
        return page;
    }

    /**
     * 各科室甲级病历占比排行（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<QcResultPercentDTO> levelPercentGroupByDeptPage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<QcResultPercentDTO> page = qcresultInfoFacade.levelPercentGroupByDeptPage(filterPageVO);
        return page;
    }

    /**
     * 条目缺陷占比（内页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<NumDTO> entryGroupByEntryInnerPage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<NumDTO> page = qcresultInfoFacade.entryGroupByEntryInnerPage(filterPageVO);
        return page;
    }
    //endregion-----------------------分页接口结束-------------------------------

    /**
     * 各科室缺陷占比（组合）
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> levelStatistics(FilterOrderVO filterOrderVO) {
        String hosptialId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterOrderVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterOrderVO.getType(), null);
        filterOrderVO.setStartDate(startDate);
        filterOrderVO.setEndDate(endDate);
        filterOrderVO.setHospitalId(hosptialId);
        List<LevelStatisticsDTO> records = behospitalInfoFacade.levelStatistics(filterOrderVO);
        return records;
    }

    /**
     * 各科室缺陷占比（组合）-全院-根据内外科系统统计
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> levelStatisticsByDeptClass(FilterOrderVO filterOrderVO) {
        String hosptialId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterOrderVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterOrderVO.getType(), null);
        filterOrderVO.setStartDate(startDate);
        filterOrderVO.setEndDate(endDate);
        filterOrderVO.setHospitalId(hosptialId);
        List<LevelStatisticsDTO> records = behospitalInfoFacade.levelStatisticsByDeptClass(filterOrderVO);
        return records;
    }

    /**
     * 病案首页合格率占比
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> homePageLevelStatistics(FilterOrderVO filterOrderVO) {
        String hosptialId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterOrderVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterOrderVO.getType(), null);
        filterOrderVO.setStartDate(startDate);
        filterOrderVO.setEndDate(endDate);
        filterOrderVO.setHospitalId(hosptialId);
        List<LevelStatisticsDTO> records = behospitalInfoFacade.homePageLevelStatistics(filterOrderVO);
        return records;
    }

    /**
     * 病案首页合格率占比
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> homePageLevelLimit(FilterOrderVO filterOrderVO) {
        DecimalFormat df = new DecimalFormat("#0.00");
        String hosptialId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterOrderVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterOrderVO.getType(), null);
        filterOrderVO.setStartDate(startDate);
        filterOrderVO.setEndDate(endDate);
        filterOrderVO.setHospitalId(hosptialId);
        if (filterOrderVO.getLimitCount() == null || filterOrderVO.getLimitCount().equals(0)) {
            filterOrderVO.setLimitCount(10);
        }
        List<LevelStatisticsDTO> records = behospitalInfoFacade.homePageLevelStatistics(filterOrderVO);

        //增加全院
        LevelStatisticsDTO item = new LevelStatisticsDTO();
        Integer mrNum = records
                .stream()
                .map(LevelStatisticsDTO::getMrNum)
                .reduce(0, Integer::sum);
        if (mrNum == null || mrNum == 0) {
            return Lists.newLinkedList();
        }
        Integer entryNum = records
                .stream()
                .map(LevelStatisticsDTO::getEntryNum)
                .reduce(0, Integer::sum);
        Double totleFee = records
                .stream()
                .map(i -> BigDecimal.valueOf(i.getTotleValue()))
                .reduce(BigDecimal.ZERO, BigDecimal::add)
                .doubleValue();
        Double averageFee = BigDecimal.valueOf(totleFee)
                .divide(BigDecimal.valueOf(mrNum), 2, RoundingMode.HALF_UP)
                .doubleValue();
        Integer firstLevelNum = records
                .stream()
                .map(LevelStatisticsDTO::getFirstLevelNum)
                .reduce(0, Integer::sum);
        Double firstLevelPercent = BigDecimal.valueOf(firstLevelNum)
                .divide(BigDecimal.valueOf(mrNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String firstLevelPercentStr
                = df.format(BigDecimal.valueOf(firstLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";
        Integer secondLevelNum = mrNum - firstLevelNum;
        Double secondLevelPercent = BigDecimal.valueOf(secondLevelNum)
                .divide(BigDecimal.valueOf(mrNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String secondLevelPercentStr
                = df.format(BigDecimal.valueOf(secondLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";
        item.setDeptName("全院");
        item.setEntryNum(entryNum);
        item.setMrNum(mrNum);
        item.setTotleValue(totleFee);
        item.setAverageValue(averageFee);
        item.setFirstLevelNum(firstLevelNum);
        item.setFirstLevelPercent(firstLevelPercent);
        item.setFirstLevelPercentStr(firstLevelPercentStr);
        item.setSecondLevelNum(secondLevelNum);
        item.setSecondLevelPercent(secondLevelPercent);
        item.setSecondLevelPercentStr(secondLevelPercentStr);

        records.add(0, item);

        records = records
                .stream()
                .limit(filterOrderVO.getLimitCount())
                .collect(Collectors.toList());

        return records;
    }
}