package com.diagbot.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.diagbot.dto.BehospitalInfoDTO;
import com.diagbot.dto.DeptNumDTO;
import com.diagbot.dto.ExportQcresultDTO;
import com.diagbot.dto.ExportExcelDTO;
import com.diagbot.dto.LevelStatisticsDTO;
import com.diagbot.dto.MsgDTO;
import com.diagbot.dto.NumDTO;
import com.diagbot.entity.BehospitalInfo;
import com.diagbot.mapper.BehospitalInfoMapper;
import com.diagbot.service.BehospitalInfoService;
import com.diagbot.vo.AnalyzeCodeVO;
import com.diagbot.vo.AnalyzeVO;
import com.diagbot.vo.BehospitalPageVO;
import com.diagbot.vo.ExportQcresultVO;
import com.diagbot.vo.FilterOrderByDeptVO;
import com.diagbot.vo.FilterOrderVO;
import com.diagbot.vo.FilterPageByDeptVO;
import com.diagbot.vo.FilterPageVO;
import com.diagbot.vo.FilterVO;
import com.diagbot.vo.TaskVO;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 住院病历信息 服务实现类
 * </p>
 *
 * @author zhoutg
 * @since 2020-04-13
 */
@Service
public class BehospitalInfoServiceImpl extends ServiceImpl<BehospitalInfoMapper, BehospitalInfo> implements BehospitalInfoService {

    @Override
    public IPage<BehospitalInfoDTO> getPage(BehospitalPageVO behospitalPageVO) {
        return baseMapper.getPage(behospitalPageVO);
    }

    @Override
    public List<MsgDTO> getMsg(AnalyzeVO analyzeVO) {
        return baseMapper.getMsg(analyzeVO);
    }

    @Override
    public List<MsgDTO> getMsgByEntryCode(AnalyzeCodeVO analyzeCodeVO) {
        return baseMapper.getMsgByEntryCode(analyzeCodeVO);
    }

    /**
     * 各科室缺陷占比-全院-首页
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<NumDTO> entryByDept(FilterVO filterVO) {
        return baseMapper.entryByDept(filterVO);
    }

    /**
     * 质控结果缺陷总数查询
     *
     * @param filterVO
     * @return
     */
    @Override
    public int getTotleResultNum(FilterVO filterVO) {
        return baseMapper.getTotleResultNum(filterVO);
    }

    @Override
    public List<BehospitalInfo> getNoGrade(TaskVO taskVO) {
        return baseMapper.getNoGrade(taskVO);
    }

    @Override
    public IPage<BehospitalInfoDTO> getPageByDept(BehospitalPageVO behospitalPageVO) {
        return baseMapper.getPageByDept(behospitalPageVO);
    }

    @Override
    public IPage<BehospitalInfoDTO> getPageByPerson(BehospitalPageVO behospitalPageVO) {
        return baseMapper.getPageByPerson(behospitalPageVO);
    }

    /**
     * 出院总人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int leaveHosCount(FilterVO filterVO) {
        return baseMapper.leaveHosCount(filterVO);
    }

    /**
     * 新生儿出院人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int newBornCount(FilterVO filterVO) {
        return baseMapper.newBornCount(filterVO);
    }

    /**
     * 死亡人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int deathCount(FilterVO filterVO) {
        return baseMapper.deathCount(filterVO);
    }

    /**
     * 手术人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int operationCount(FilterVO filterVO) {
        return baseMapper.operationCount(filterVO);
    }

    /**
     * 出院总人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> leaveHosCountByDept(FilterVO filterVO) {
        return baseMapper.leaveHosCountByDept(filterVO);
    }

    /**
     * 新生儿出院人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> newBornCountByDept(FilterVO filterVO) {
        return baseMapper.newBornCountByDept(filterVO);
    }

    /**
     * 死亡人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> deathCountByDept(FilterVO filterVO) {
        return baseMapper.deathCountByDept(filterVO);
    }

    /**
     * 手术人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> operationCountByDept(FilterVO filterVO) {
        return baseMapper.operationCountByDept(filterVO);
    }

    /**
     * 各科室缺陷占比排行（分页）
     *
     * @param filterPageVO
     * @return
     */
    @Override
    public IPage<NumDTO> resultStatisticsByDeptPage(@Param("filterPageVO") FilterPageVO filterPageVO) {
        return baseMapper.resultStatisticsByDeptPage(filterPageVO);
    }

    /**
     * 各科室缺陷占比（组合）
     *
     * @param filterOrderVO
     * @return
     */
    @Override
    public List<LevelStatisticsDTO> levelStatistics(FilterOrderVO filterOrderVO) {
        return baseMapper.levelStatistics(filterOrderVO);
    }

    /**
     * 病案首页合格率占比
     *
     * @param filterOrderVO
     * @return
     */
    @Override
    public List<LevelStatisticsDTO> homePageLevelStatistics(FilterOrderVO filterOrderVO) {
        return baseMapper.homePageLevelStatistics(filterOrderVO);
    }

    /**
     * 各科室缺陷占比（主任医生）
     *
     * @param filterOrderByDeptVO
     * @return
     */
    @Override
    public List<LevelStatisticsDTO> levelStatisticsByDept(FilterOrderByDeptVO filterOrderByDeptVO) {
        return baseMapper.levelStatisticsByDept(filterOrderByDeptVO);
    }

    /**
     * 病案首页合格率占比（主任医生）
     *
     * @param filterOrderByDeptVO
     * @return
     */
    @Override
    public List<LevelStatisticsDTO> homePageLevelStatisticsByDept(FilterOrderByDeptVO filterOrderByDeptVO) {
        return baseMapper.homePageLevelStatisticsByDept(filterOrderByDeptVO);
    }

    /**
     * 缺陷明细-科室（分页）
     *
     * @param filterPageByDeptVO
     * @return
     */
    public IPage<DeptNumDTO> resultStatisticsByDeptAndDoctorPage(@Param("filterPageByDeptVO") FilterPageByDeptVO filterPageByDeptVO) {
        return baseMapper.resultStatisticsByDeptAndDoctorPage(filterPageByDeptVO);
    }

    /**
     * 各科室缺陷占比（组合）-全院-根据内外科系统统计
     *
     * @param filterOrderVO
     * @return
     */
    @Override
    public List<LevelStatisticsDTO> levelStatisticsByDeptClass(FilterOrderVO filterOrderVO) {
        return baseMapper.levelStatisticsByDeptClass(filterOrderVO);
    }

    @Override
    public List<ExportExcelDTO> exportExcel(){
        return baseMapper.exportExcel();
    }

    /**
     * 质控评分导出到excel
     *
     * @param exportQcresultVO
     * @return
     */
    public List<ExportExcelDTO> exportQcresult(ExportQcresultVO exportQcresultVO) {
        return baseMapper.exportQcresult(exportQcresultVO);
    }

    /**
     * 科室质控评分导出到excel
     *
     * @param exportQcresultVO
     * @return
     */
    public List<ExportExcelDTO> exportQcresultByDept(ExportQcresultVO exportQcresultVO){
        return baseMapper.exportQcresultByDept(exportQcresultVO);
    }
}